@@ -2,17 +2,25 @@ import json
 
 from pkg_resources import resource_filename
 
-CORE_TABLE_PATH = resource_filename(__package__, "iso-639-3.tab")
-PART2_TABLE_PATH = resource_filename(__package__, "ISO-639-2_utf-8.txt")
-PART5_TABLE_PATH = resource_filename(__package__, "iso639-5.tsv")
-MAPPING_PATH = resource_filename(__package__, "iso-639.json")
-
+FILENAMES = {
+    "core": "iso-639-3.tab",
+    "pt2": "ISO-639-2_utf-8.txt",
+    "pt5": "iso639-5.tsv",
+    "macro": "iso-639-3-macrolanguages.tab",
+    "mapping": "iso-639.json",
+    "mapping_macro": "iso-639_macro.json",
+}
 TAGS = ("pt1", "pt2b", "pt2t", "pt3", "pt5", "name")
 
 
 class ISO639Mapping:
     """A mapping used to query the ISO 639 values efficiently"""
 
+    _fps = {
+        k: resource_filename(__package__, FILENAMES[k])
+        for k in ("core", "pt2", "pt5", "mapping")
+    }
+
     def __init__(self):
 
         self._data = {}
@@ -20,7 +28,7 @@ class ISO639Mapping:
     def load(self):
         """Load data from local JSON file"""
         try:
-            with open(MAPPING_PATH, encoding="utf-8") as f:
+            with open(self._fps["mapping"], encoding="utf-8") as f:
                 self._data = json.load(f)
         except FileNotFoundError:
             self.build()
@@ -35,23 +43,24 @@ class ISO639Mapping:
 
     def save(self):
         """Save data into local JSON file"""
-        with open(MAPPING_PATH, "w", encoding="utf-8") as f:
+        with open(self._fps["mapping"], "w", encoding="utf-8") as f:
             json.dump(self._data, f)
 
     def build(self):
         """Build the JSON file from the official ISO 639 code tables
         stored locally
         """
-        self._data = self._build_core_mapping()
-        self._complete_part2()
-        self._add_part5()
-        self._sort_alphabetically()
+        mapping = self._build_core_mapping()
+        mapping = self._complete_part2(mapping)
+        mapping = self._add_part5(mapping)
 
-    @staticmethod
-    def _build_core_mapping():
+        self._data = self._sort_alphabetically(mapping)
+
+    @classmethod
+    def _build_core_mapping(cls):
         """Builds the core ISO-639 mapping from the ISO 639-3 tab file"""
         mapping = {}
-        with open(CORE_TABLE_PATH, encoding="utf-8") as f:
+        with open(cls._fps["core"], encoding="utf-8") as f:
             next(f)
             for line in f:
                 row = line.rstrip().split("\t")
@@ -71,9 +80,10 @@ class ISO639Mapping:
 
         return mapping
 
-    def _complete_part2(self):
+    @classmethod
+    def _complete_part2(cls, mapping):
         """Complete mapping with pt2 codes not covered by pt3"""
-        with open(PART2_TABLE_PATH, encoding="utf-8-sig") as f:
+        with open(cls._fps["pt2"], encoding="utf-8-sig") as f:
             for line in f:
                 row = line.rstrip().split("|")
                 params = {
@@ -86,14 +96,16 @@ class ISO639Mapping:
                 }
                 if len(params["pt2b"]) == 3:
                     for tag in TAGS:
-                        if params[tag] and params[tag] not in self._data[tag]:
-                            self._data.setdefault(tag, {})[params[tag]] = {
+                        if params[tag] and params[tag] not in mapping[tag]:
+                            mapping.setdefault(tag, {})[params[tag]] = {
                                 k: v for k, v in params.items() if k != tag
                             }
+        return mapping
 
-    def _add_part5(self):
+    @classmethod
+    def _add_part5(cls, mapping):
         """Add ISO 639-2 pt5 codes to mapping"""
-        with open(PART5_TABLE_PATH, encoding="utf-8") as f:
+        with open(cls._fps["pt5"], encoding="utf-8") as f:
             next(f)
             for line in f:
                 row = line.rstrip().split("\t")
@@ -107,21 +119,77 @@ class ISO639Mapping:
                 }
                 if len(params["pt5"]) == 3:
                     for tag in TAGS:
-                        if params[tag] and params[tag] not in self._data.get(
+                        if params[tag] and params[tag] not in mapping.get(
                             tag, {}
                         ):
-                            self._data.setdefault(tag, {})[params[tag]] = {
+                            mapping.setdefault(tag, {})[params[tag]] = {
                                 k: v for k, v in params.items() if k != tag
                             }
                 # map pt5 and pt2b
-                if params["pt5"] in self._data["pt2b"]:
-                    self._data["pt2b"][params["pt5"]]["pt5"] = params["pt5"]
-                    self._data["pt5"][params["pt5"]]["pt2b"] = params["pt5"]
+                if params["pt5"] in mapping["pt2b"]:
+                    mapping["pt2b"][params["pt5"]]["pt5"] = params["pt5"]
+                    mapping["pt5"][params["pt5"]]["pt2b"] = params["pt5"]
 
-    def _sort_alphabetically(self):
+        return mapping
+
+    @staticmethod
+    def _sort_alphabetically(mapping):
         """Sort language codes and names alphabetically in the mapping"""
         sorted_data = {}
-        for tag, d1 in self._data.items():
+        for tag, d1 in mapping.items():
             sorted_data[tag] = {lg: d2 for lg, d2 in sorted(d1.items())}
 
-        self._data = {k: v for k, v in sorted(sorted_data.items())}
+        mapping = {k: v for k, v in sorted(sorted_data.items())}
+
+        return mapping
+
+
+class MacroMapping:
+    """A mapping used to query the ISO 639 macrolanguages efficiently"""
+
+    _fps = {
+        k: resource_filename(__package__, FILENAMES[k])
+        for k in ("macro", "mapping_macro")
+    }
+
+    def __init__(self):
+
+        self._data = {}
+
+    def load(self):
+        """Load data from local JSON file"""
+        try:
+            with open(self._fps["mapping_macro"], encoding="utf-8") as f:
+                self._data = json.load(f)
+        except FileNotFoundError:
+            mapping = self._build()
+            self._data = self._sort_alphabetically(mapping)
+            self.save()
+
+        return self._data
+
+    def save(self):
+        """Save data into local JSON file"""
+        with open(self._fps["mapping_macro"], "w", encoding="utf-8") as f:
+            json.dump(self._data, f)
+
+    @classmethod
+    def _build(cls):
+
+        mapping = {k: {} for k in ("macro", "individual")}
+        with open(cls._fps["macro"], encoding="utf-8") as f:
+            next(f)
+            for line in f:
+                row = line.rstrip().split("\t")
+                mapping["macro"].setdefault(row[0], []).append(row[1])
+                mapping["individual"][row[1]] = row[0]
+
+        return mapping
+
+    @staticmethod
+    def _sort_alphabetically(mapping):
+
+        mapping["individual"] = {
+            k: v for k, v in sorted(mapping["individual"].items())
+        }
+        return mapping
